from aiohttp import ClientSession
from aiohttp import ClientResponseError

from camiskaze.adapters.exceptions import RegistrationError
from camiskaze.adapters.models import CamisTimeEntry
from camiskaze.domain.messages import RegisterTimeEntry
from camiskaze.domain.util import to_nearest_quarter


class CamisTimeEntryRepository:
    _url = "api_camis/timesheet/"

    def __init__(self, session: ClientSession):
        self.session = session

    async def register(self, command: RegisterTimeEntry) -> None:
        record = _to_record(command)
        await self._register(record)

    async def _register(self, record: CamisTimeEntry) -> None:
        data = record.model_dump_json()

        try:
            async with self.session.post(self._url, data=data) as response:
                response.raise_for_status()
        except ClientResponseError as error:
            raise RegistrationError(str(error))


def _to_record(command: RegisterTimeEntry) -> CamisTimeEntry:
    time_entry = command.time_entry

    return CamisTimeEntry(
        userid=command.user,
        description=time_entry.description,
        workorder=time_entry.work_order,
        registration_date=time_entry.date,
        hour_count=time_entry.duration_hours,
    )
