from rich.console import Console
from rich.status import Status


_DEFAULT_COLOR = "bold green"
_DEFAULT_SPINNER = "clock"
_DEFAULT_TEXT = "Retrieving Time Entries from Toggl"
_DEFAULT_TIME_FORMAT = "[%Y-%m-%d %H:%M:%S.%f]"


class StatusAdapter:
    status: Status

    def __init__(
        self,
        console: Console = None,
        color: str = _DEFAULT_COLOR,
        text: str = _DEFAULT_TEXT,
        spinner: str = _DEFAULT_SPINNER,
    ):
        self.console = console or Console(log_time_format=_DEFAULT_TIME_FORMAT)
        self.color = color
        self.text = text
        self.spinner = spinner

        self.status = None

    def start(self, message: str) -> None:
        self.status = self.console.status(
            status=f"[{self.color}]{message}",
            spinner="clock",
        )

        self.status.start()

    def log(self, message: str) -> None:
        self.console.log(message)

    def stop(self) -> None:
        self.status.stop()
