from rich.align import Align
from rich.console import Console
from rich.table import Table

from camiskaze.domain.messages import TimesheetRetrieved
from camiskaze.domain.messages.events import (
    TimeEntryRegistered,
    TimesheetRegistered,
)


_TITLE_TEMPLATE = "Registration from {start} to {end} for {user}."


class TableAdapter:
    table: Table

    def __init__(self, console: Console = None):
        self.console = console or Console()

        self._caption_set = False
        self.table = Table(
            padding=(0, 2),
            show_edge=False,
            row_styles=["dim", ""],
            caption="[not italic]:calendar:[/] CamisKaze [not italic]:mantelpiece_clock:[/]",
        )
        Align.center(self.table)

        self.__init__columns()

    def __init__columns(self) -> None:
        self.table.add_column("Result", no_wrap=True, justify="center")
        self.table.add_column(
            "Project",
            no_wrap=True,
            justify="left",
            style="dark_orange3",
            header_style="bold dark_orange3",
        )
        self.table.add_column(
            "Description",
            no_wrap=True,
            justify="left",
            style="cyan",
            header_style="bold cyan",
        )
        self.table.add_column(
            "Duration",
            no_wrap=True,
            justify="left",
            style="blue",
            header_style="bold blue",
        )
        self.table.add_column(
            "Date",
            no_wrap=True,
            justify="left",
            style="green",
            header_style="bold green",
        )
        self.table.add_column(
            "Work Order",
            no_wrap=True,
            justify="left",
            style="yellow",
            header_style="bold yellow",
        )
        self.table.add_column(
            "Reason",
            no_wrap=True,
            justify="left",
            style="red",
            header_style="bold red",
        )

    def add(self, event: TimeEntryRegistered) -> None:
        if event.success:
            success = ":white_check_mark:"
        else:
            success = ":skull:"

        self.table.add_row(
            success,
            event.project,
            event.description,
            str(event.duration_hours),
            event.date.isoformat(),
            event.work_order,
            event.reason,
        )

    def print(self, event: TimesheetRegistered) -> None:
        self.console.print(self.table, new_line_start=True)

    def set_title(self, event: TimesheetRetrieved) -> None:
        title = _TITLE_TEMPLATE.format(
            start=event.start,
            end=event.end,
            user=event.user,
        )
        self.table.title = title
