from aiohttp import ClientSession

from camiskaze.adapters.models import TogglProject


class TogglProjectRepository:
    _url_template = "/api/v9/workspaces/{workspace_id}/projects/{project_id}"

    def __init__(self, session: ClientSession):
        self.session = session

    async def get(self, workspace_id: int, project_id: int) -> TogglProject:
        record = await self._get(workspace_id, project_id)

        return TogglProject.model_validate(record)

    async def _get(self, workspace_id: int, project_id: int) -> dict:
        url = self._build_url(workspace_id, project_id)

        async with self.session.get(url) as response:
            response.raise_for_status()

            return await response.json()

    def _build_url(self, workspace_id: int, project_id: int) -> str:
        return self._url_template.format(
            workspace_id=workspace_id,
            project_id=project_id,
        )
