from datetime import date, timedelta
from typing import AsyncGenerator

from aiohttp import ClientSession

from camiskaze.config import DATE_FORMAT
from camiskaze.domain.models import TimeEntry

from . import TogglWorkOrderRepository


class TogglTimeEntryRepository:
    _url = "/api/v9/me/time_entries"
    _work_orders: TogglWorkOrderRepository

    def __init__(self, session: ClientSession):
        self.session = session
        self._work_orders = TogglWorkOrderRepository(session)

        self.seen = []

    async def list(
        self,
        start: date,
        end: date,
    ) -> AsyncGenerator[TimeEntry]:
        async for record in self._list(start, end):
            workspace_id = record["workspace_id"]
            project_id = record["project_id"]
            work_order, project = await self._work_orders.get(
                workspace_id,
                project_id,
            )
            yield TimeEntry(
                description=record["description"],
                work_order=work_order or "",
                start=record["start"],
                duration=record["duration"],
                project=project,
            )

    async def _list(self, start: date, end: date) -> AsyncGenerator[dict]:
        end += timedelta(days=1)
        async with self.session.get(
            self._url,
            params={
                "start_date": start.strftime(DATE_FORMAT),
                "end_date": end.strftime(DATE_FORMAT),
            },
        ) as response:
            response.raise_for_status()

            for record in await response.json():
                yield record
