from aiohttp import ClientSession

from camiskaze.domain.messages import RetrieveTimesheet
from camiskaze.domain.models import TimeEntry, Timesheet

from . import TogglTimeEntryRepository


class TogglTimeSheetRepository:
    _time_entries: TogglTimeEntryRepository

    def __init__(self, session: ClientSession):
        self._time_entries = TogglTimeEntryRepository(session)

    async def get(self, command: RetrieveTimesheet) -> Timesheet:
        start = command.start
        end = command.end

        entries = [
            entry async for entry in self._time_entries.list(start, end)
        ]

        return Timesheet(
            user=command.user,
            start=start,
            end=end,
            entries=entries,
        )

    @property
    def seen(self) -> list[TimeEntry]:
        return self._time_entries.seen
