import datetime

from pydantic import BaseModel

from camiskaze.domain.util import to_nearest_quarter


class TimeEntry(BaseModel):
    description: str
    work_order: str
    start: datetime.datetime
    duration: float
    project: str = ""

    @property
    def duration_hours(self) -> float:
        return to_nearest_quarter(self.duration)

    @property
    def date(self) -> datetime.date:
        return self.start.date()

    def __hash__(self) -> str:
        return hash(self._unique_fields)

    def __eq__(self, other: "TimeEntry") -> bool:
        return self._unique_fields == other._unique_fields

    @property
    def _unique_fields(self) -> tuple:
        return (
            self.description,
            self.work_order,
            self.start.isoformat(),
            self.duration,
        )
