from collections.abc import Generator
from datetime import datetime

from pydantic import BaseModel

from . import TimeEntry


class Timesheet(BaseModel):
    user: str
    start: datetime
    end: datetime
    entries: list[TimeEntry]

    def list(self) -> Generator[TimeEntry]:
        yield from self.entries

    @property
    def num_entries(self) -> int:
        return len(self.entries)

    def __hash__(self) -> str:
        return hash((self.user, self.start, self.end))

    def __eq__(self, other: "Timesheet") -> bool:
        if self.user != other.user:
            return False
        if self.start != other.start:
            return False
        if self.end != other.end:
            return False

        return True
