from collections import defaultdict, namedtuple

from camiskaze.domain.models import Timesheet, TimeEntry


GroupKey = namedtuple(
    "GroupKey",
    ["project", "date", "description", "work_order"],
)


def group(timesheet: Timesheet) -> None:
    grouped_entries = defaultdict(int)
    for entry in timesheet.entries:
        grouped_entries[_to_group_key(entry)] += entry.duration

    timesheet.entries = [
        _to_new_entry(entry, total_duration)
        for entry, total_duration in grouped_entries.items()
    ]


def _to_group_key(entry: TimeEntry) -> GroupKey:
    return GroupKey(
        date=entry.date,
        description=entry.description,
        work_order=entry.work_order,
        project=entry.project,
    )


def _to_new_entry(key: GroupKey, total_duration: float) -> TimeEntry:
    return TimeEntry(
        project=key.project,
        description=key.description,
        work_order=key.work_order,
        start=key.date,
        duration=total_duration,
    )
