from datetime import datetime

from asyncclick import (
    DateTime,
    STRING,
    group,
    version_option,
    option,
)

from camiskaze.config import DATE_FORMAT, DATE_FORMAT_HUMAN_READABLE
from camiskaze.config.cli_config import BANNER, HELP_TEXT

from .models import BannerGroup


@group(
    cls=BannerGroup,
    banner=BANNER,
    help=HELP_TEXT,
)
@version_option("0.1.3")
async def cli() -> None:
    from camiskaze.bootstrap import setup

    await setup()


@cli.command(help="Register your timesheets from Toggl.")
@option(
    "--camis-user",
    type=STRING,
    required=True,
    help="Your Camis (i.e. AD) user's ID.",
)
@option(
    "--toggl-user",
    type=STRING,
    required=True,
    help="Your Toggl username.",
)
@option(
    "-s",
    "--start",
    type=DateTime(formats=[DATE_FORMAT]),
    required=True,
    help=f"Start date in {DATE_FORMAT_HUMAN_READABLE} format.",
)
@option(
    "-e",
    "--end",
    type=DateTime(formats=[DATE_FORMAT]),
    required=True,
    help=f"End date in {DATE_FORMAT_HUMAN_READABLE} format.",
)
async def register(
    camis_user: str,
    toggl_user: str,
    start: datetime,
    end: datetime,
) -> None:
    from camiskaze.bootstrap import create_message_bus, create_unit_of_work
    from camiskaze.domain.messages import Register

    message_bus = create_message_bus()

    command = Register(
        camis_user=camis_user,
        toggl_user=toggl_user,
        start=start,
        end=end,
    )
    unit_of_work = create_unit_of_work(toggl_user)

    await message_bus.publish(message=command, unit_of_work=unit_of_work)
