from camiskaze.domain.messages.commands import (
    Register,
    RetrieveTimesheet,
    RegisterTimesheet,
)
from camiskaze.domain.models import Timesheet
from camiskaze.service_layer import UnitOfWork, MessageBus


async def register(
    command: Register,
    unit_of_work: UnitOfWork,
    message_bus: MessageBus,
) -> None:
    async with unit_of_work as uow:
        await _register(command, uow, message_bus)


async def _register(
    command: Register,
    uow: UnitOfWork,
    message_bus: MessageBus,
) -> None:
    timesheet = await _retrieve_timesheet(command, uow, message_bus)
    await _register_timesheet(command, uow, message_bus, timesheet)


async def _retrieve_timesheet(
    command: Register,
    uow: UnitOfWork,
    message_bus: MessageBus,
) -> Timesheet:
    command = RetrieveTimesheet(
        user=command.toggl_user,
        start=command.start,
        end=command.end,
    )
    return await message_bus.publish(command, uow)


async def _register_timesheet(
    command: Register,
    uow: UnitOfWork,
    message_bus: MessageBus,
    timesheet: Timesheet,
) -> None:
    command = RegisterTimesheet(
        timesheet=timesheet,
        camis_user=command.camis_user,
    )
    await message_bus.publish(command, uow)
