from camiskaze.domain.messages.commands import (
    RegisterTimesheet,
    RegisterTimeEntry,
)
from camiskaze.domain.messages.events import TimesheetRegistered
from camiskaze.domain.models import TimeEntry
from camiskaze.service_layer import UnitOfWork, MessageBus


async def register_timesheet(
    command: RegisterTimesheet,
    unit_of_work: UnitOfWork,
    message_bus: MessageBus,
) -> None:
    await _register_timesheet(command, unit_of_work, message_bus)

    await _log_completion(unit_of_work, message_bus)


async def _register_timesheet(
    command: RegisterTimesheet,
    unit_of_work: UnitOfWork,
    message_bus: MessageBus,
) -> None:
    camis_user = command.camis_user

    for time_entry in command.time_entries:
        await _register_time_entry(
            camis_user,
            time_entry,
            unit_of_work,
            message_bus,
        )


async def _register_time_entry(
    camis_user: str,
    time_entry: TimeEntry,
    unit_of_work: UnitOfWork,
    message_bus: MessageBus,
) -> None:
    command = RegisterTimeEntry(user=camis_user, time_entry=time_entry)

    return await message_bus.publish(command, unit_of_work)


async def _log_completion(
    unit_of_work: UnitOfWork,
    message_bus: MessageBus,
):
    event = TimesheetRegistered(success=True)

    await message_bus.publish(event, unit_of_work)
