from camiskaze.domain.messages import (
    RetrieveTimesheet,
    TimesheetRetrieved,
    TimesheetRetrievalStarted,
)
from camiskaze.domain.models import Timesheet
from camiskaze.service_layer import UnitOfWork, MessageBus


async def retrieve_timesheet(
    command: RetrieveTimesheet,
    unit_of_work: UnitOfWork,
    message_bus: MessageBus,
) -> Timesheet:
    await _log_start(command, unit_of_work, message_bus)
    timesheet = await unit_of_work.toggl.get(command)
    await _log_completion(timesheet, unit_of_work, message_bus)

    return timesheet


async def _log_start(
    command: RetrieveTimesheet,
    unit_of_work: UnitOfWork,
    message_bus: MessageBus,
) -> None:
    log_event = TimesheetRetrievalStarted(
        start=command.start,
        end=command.end,
    )
    await message_bus.publish(log_event, unit_of_work)


async def _log_completion(
    timesheet: Timesheet,
    unit_of_work: UnitOfWork,
    message_bus: MessageBus,
) -> None:
    log_event = TimesheetRetrieved(timesheet=timesheet)
    await message_bus.publish(log_event, unit_of_work)
