from camiskaze.domain import timesheet_grouper
from camiskaze.domain.messages import TimesheetGrouped, TimesheetRetrieved
from camiskaze.service_layer import UnitOfWork, MessageBus


_MSG_TEMPLATE = "Retrieved {num} Time Entries."


async def log_retrieval(
    event: TimesheetRetrieved,
    unit_of_work: UnitOfWork,
    _: MessageBus,
) -> None:
    msg = _MSG_TEMPLATE.format(num=event.num_time_entries)
    unit_of_work.status.log(msg)


async def start_progress(
    event: TimesheetRetrieved,
    unit_of_work: UnitOfWork,
    _: MessageBus,
) -> None:
    unit_of_work.progress.start(event)


async def set_table_title(
    event: TimesheetRetrieved,
    unit_of_work: UnitOfWork,
    _: MessageBus,
) -> None:
    unit_of_work.table.set_title(event)


async def group_timesheet(
    event: TimesheetRetrieved,
    unit_of_work: UnitOfWork,
    message_bus: MessageBus,
) -> None:
    initial_num_entries = event.num_time_entries
    timesheet_grouper.group(event.timesheet)

    grouped_event = TimesheetGrouped(
        timesheet=event.timesheet,
        initial_num_entries=initial_num_entries,
        result_num_entries=event.num_time_entries,
    )
    await message_bus.publish(grouped_event, unit_of_work)
