import asyncio
from typing import Any, Callable, Type

from camiskaze.domain.messages import Command, Message

from . import MessageHandlers, UnitOfWork


class MessageBus:
    handlers: MessageHandlers = MessageHandlers()

    def register(self, message_type: Type, handler: Callable) -> None:
        self.handlers.register(message_type, handler)

    async def publish(
        self,
        message: Message,
        unit_of_work: UnitOfWork,
    ) -> None:
        if isinstance(message, Command):
            return await self.handle_command(message, unit_of_work)

        await self.handle_event(message, unit_of_work)

    async def handle_command(
        self,
        message: Message,
        unit_of_work: UnitOfWork,
    ) -> Any:
        handler = self.handlers.get(message)

        return await handler(message, unit_of_work, self)

    async def handle_event(
        self,
        message: Message,
        unit_of_work: UnitOfWork,
    ) -> None:
        tasks = [
            asyncio.create_task(handler(message, unit_of_work, self))
            for handler in self.handlers.get(message)
        ]
        await asyncio.gather(*tasks)
