from collections import defaultdict
from typing import Type, Callable

from pydantic import BaseModel

from camiskaze.domain.messages import Command, Event, Message


class MessageHandlers(BaseModel):
    command_handlers: dict[Type[Command], Callable] = {}
    event_handlers: dict[Type[Event], list[Callable]] = defaultdict(list)

    def get(self, message: Message) -> list[Callable] | Callable:
        if isinstance(message, Command):
            return self.command_handlers[type(message)]
        else:
            return self.event_handlers[type(message)]

    def register(self, message_type: Type, handler: Callable) -> None:
        if issubclass(message_type, Command):
            self.command_handlers[message_type] = handler
        else:
            self.event_handlers[message_type].append(handler)
